@@ -22,6 +22,7 @@
  *   Stuart Parmenter <stuart.parmenter@oracle.com>
  *   Michael Buettner <michael.buettner@sun.com>
  *   Stefan Sitter <ssitter@gmail.com>
+ *   Philipp Kewisch <mozilla@kewis.ch>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -37,104 +38,6 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-/* utility functions */
-
-function setElementValue(elementName, value, name)
-{
-    var element = document.getElementById(elementName);
-    if (!element) {
-        dump("unable to find " + elementName + "\n");
-        return;
-    }
-
-    if (value === false) {
-        element.removeAttribute(name ? name : "value");
-    } else if (name) {
-        //dump("element.setAttribute(" + name + ", " + value + ")\n");
-        element.setAttribute(name, value);
-    } else {
-        //dump("element.value = " + value + "\n");
-        element.value = value;
-    }
-}
-
-function getElementValue(elementName, name)
-{
-    var element = document.getElementById(elementName);
-    if (!element) {
-        dump("unable to find " + elementName + "\n");
-        return null;
-    }
-
-    if (name)
-        return element[name];
-
-    return element.value;
-}
-
-function enableElement(elementId)
-{
-    setElementValue(elementId, false, "disabled");
-}
-
-function disableElement(elementId)
-{
-    setElementValue(elementId, "true", "disabled");
-}
-
-/**
- * This function unconditionally disables the element for
- * which the id has been passed as argument. Furthermore, it
- * remembers who was responsible for this action by using
- * the given key (lockId). In case the control should be
- * enabled again the lock gets removed, but the control only
- * gets enabled if *all* possibly held locks have been removed.
- */
-function disableElementWithLock(elementId,lockId) {
-
-    // unconditionally disable the element.
-    disableElement(elementId);
-
-    // remember that this element has been locked with
-    // the key passed as argument. we keep a primitive
-    // form of ref-count in the attribute 'lock'.
-    var element = document.getElementById(elementId);
-    if (element) {
-        if (!element.hasAttribute(lockId)) {
-            element.setAttribute(lockId, "true");
-            var n = parseInt(element.getAttribute("lock") || 0);
-            element.setAttribute("lock", n + 1);
-        }
-    }
-}
-
-/**
- * This function is intended to be used in tandem with the
- * above defined function 'disableElementWithLock()'.
- * See the respective comment for further details.
- */
-function enableElementWithLock(elementId, lockId) {
-
-    var element = document.getElementById(elementId);
-    if (!element) {
-        dump("unable to find " + elementId + "\n");
-        return;
-    }
-
-    if (element.hasAttribute(lockId)) {
-        element.removeAttribute(lockId);
-        var n = parseInt(element.getAttribute("lock") || 0) - 1;
-        if (n > 0) {
-            element.setAttribute("lock", n);
-        } else {
-            element.removeAttribute("lock");
-        }
-        if (n <= 0) {
-            enableElement(elementId);
-        }
-    }
-}
-
 /* use with textfields oninput to only allow integers */
 function validateIntegerRange(event, lowerBound, upperBound) {
     validateIntegers(event);
@@ -170,20 +73,8 @@ function validateNaturalNums(event) {
 /**
  * This function takes the recurrence info passed as argument and creates a
  * literal string representing the repeat pattern in natural language.
- * It expects a xul <box> with the id 'repeat-details' which should hold
- * at least a single <label> element. The structure should look similar to
- * this example:
- *                     <vbox id="repeat-details">
- *                       <label/>
- *                     </vbox>
  */
-function commonUpdateRepeatDetails(recurrenceInfo, startDate, endDate, allDay) {
-    // First of all collapse the details text. If we fail to
-    // create a details string, we simply don't show anything.
-    // this could happen if the repeat rule is something exotic
-    // we don't have any strings prepared for.
-    var repeatDetails = document.getElementById("repeat-details");
-    repeatDetails.setAttribute("collapsed", "true");
+function recurrenceRule2String(recurrenceInfo, startDate, endDate, allDay) {
 
     // Retrieve a valid recurrence rule from the currently
     // set recurrence info. Bail out if there's more
@@ -339,7 +230,7 @@ function commonUpdateRepeatDetails(recurrenceInfo, startDate, endDate, allDay) {
                         // freeze for 0.7 so I can't add the necessary bits and
                         // pieces.
                         if (component[i] < 0) {
-                            return;
+                            return null;
                         }
                         day_string += component[i];
                         if (component.length > 1 &&
@@ -501,7 +392,7 @@ function commonUpdateRepeatDetails(recurrenceInfo, startDate, endDate, allDay) {
                 .getService(Components.interfaces.calIDateTimeFormatter);
 
             var detailsString;
-            if (allDay) {
+            if (!endDate || allDay) {
                 if (rule.isFinite) {
                     if (rule.isByCount) {
                         detailsString = calGetString(
@@ -559,25 +450,10 @@ function commonUpdateRepeatDetails(recurrenceInfo, startDate, endDate, allDay) {
                           dateFormatter.formatTime(endDate) ], 4);
                 }
             }
-
-            if (detailsString) {
-                var lines = detailsString.split("\n");
-                repeatDetails.removeAttribute("collapsed");
-                while (repeatDetails.childNodes.length > lines.length) {
-                    repeatDetails.removeChild(repeatDetails.lastChild);
-                }
-                var numChilds = repeatDetails.childNodes.length;
-                for (var i = 0; i < lines.length; i++) {
-                    if (i >= numChilds) {
-                        var newNode = repeatDetails.childNodes[0]
-                                                   .cloneNode(true);
-                        repeatDetails.appendChild(newNode);
-                    }
-                    repeatDetails.childNodes[i].value = lines[i];
-                }
-            }
+            return detailsString;
         }
     }
+    return null;
 }
 
 function splitRecurrenceRules(recurrenceInfo) {
