@@ -39,6 +39,10 @@
 
 const kHoursBetweenUpdates = 6;
 
+function nowUTC() {
+    return jsDateToDateTime(new Date()).getInTimezone(UTC());
+}
+
 function newTimerWithCallback(callback, delay, repeating)
 {
     var timer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
@@ -84,10 +88,7 @@ function calAlarmService() {
             } else {
                 occs = [aItem];
             }
-            function hasAlarm(a) {
-                return a.alarmOffset || a.parentItem.alarmOffset;
-            }
-            occs = occs.filter(hasAlarm);
+
             for each (var occ in occs) {
                 this.alarmService.addAlarm(occ);
             }
@@ -97,17 +98,33 @@ function calAlarmService() {
                 // deleting an occurrence currently calls modifyItem(newParent, *oldOccurrence*)
                 aOldItem = aOldItem.parentItem;
             }
-            this.alarmService.removeAlarm(aOldItem);
 
+            this.onDeleteItem(aOldItem);
             this.onAddItem(aNewItem);
         },
         onDeleteItem: function(aDeletedItem) {
             this.alarmService.removeAlarm(aDeletedItem);
         },
-        onError: function(aErrNo, aMessage) { }
+        onError: function(aErrNo, aMessage) {},
+        onPropertyChanged: function(aCalendar, aName, aValue, aOldValue) {
+            if (aName == "suppressAlarms") {
+                // While in the UI it should be assured that suppressAlarms is
+                // not changed if popup alarms are unsupported, be robust here.
+                if ((!aOldValue && aValue) ||
+                    aCalendar.getProperty("capabilities.alarms.popup.supported") === false) {
+                    this.alarmService.notifyObservers("onRemoveAlarmsByCalendar", [aCalendar]);
+                } else if (aOldValue && !aValue) {
+                    this.alarmService.initAlarms([aCalendar]);
+                }
+            }
+        },
+        onPropertyDeleting: function(aCalendar, aName) {
+            if (aName == "suppressAlarms") {
+                this.onPropertyChanged(aCalendar, aName, null, aCalendar.getProperty(aName));
+            }
+        }
     };
 
-
     this.calendarManagerObserver = {
         alarmService: this,
 
@@ -122,9 +139,7 @@ function calAlarmService() {
             // dismissing those alarms may write data...
             this.alarmService.unobserveCalendar(aCalendar);
         },
-        onCalendarDeleting: function(aCalendar) {},
-        onCalendarPrefChanged: function(aCalendar, aName, aValue, aOldValue) {},
-        onCalendarPrefDeleting: function(aCalendar, aName) {}
+        onCalendarDeleting: function(aCalendar) {}
     };
 }
 
@@ -172,8 +187,9 @@ calAlarmService.prototype = {
         return this;
     },
 
-
     /* nsIObserver */
+    // This will also be called on app-startup, but nothing is done yet, to
+    // prevent unwanted dialogs etc. See bug 325476 and 413296 
     observe: function cas_observe(subject, topic, data) {
         if (topic == "profile-after-change") {
             this.shutdown();
@@ -191,7 +207,7 @@ calAlarmService.prototype = {
     },
 
     set timezone(aTimezone) {
-        this.mTimezone = aTimezone;
+        return (this.mTimezone = aTimezone);
     },
 
     snoozeAlarm: function cas_snoozeAlarm(event, duration) {
@@ -199,7 +215,7 @@ calAlarmService.prototype = {
         // Make sure we're working with the parent, otherwise we'll accidentally
         // create an exception
         var newEvent = event.parentItem.clone();
-        var alarmTime = jsDateToDateTime((new Date())).getInTimezone("UTC");
+        var alarmTime = nowUTC();
 
         // Set the last acknowledged time to now.
         newEvent.alarmLastAck = alarmTime;
@@ -223,7 +239,7 @@ calAlarmService.prototype = {
     },
 
     dismissAlarm: function cas_dismissAlarm(item) {
-        var now = jsDateToDateTime(new Date()).getInTimezone("UTC");
+        var now = nowUTC();
         // We want the parent item, otherwise we're going to accidentally create an
         // exception.  We've relnoted (for 0.1) the slightly odd behavior this can
         // cause if you move an event after dismissing an alarm
@@ -251,10 +267,6 @@ calAlarmService.prototype = {
         this.mObservers.notify(functionName, args);
     },
 
-    hasAlarm: function cas_hasAlarm(aItem) {
-        return aItem.alarmOffset || aItem.parentItem.alarmOffset;
-    },
-
     startup: function cas_startup() {
         if (this.mStarted)
             return;
@@ -293,7 +305,7 @@ calAlarmService.prototype = {
         var timerCallback = {
             alarmService: this,
             notify: function timer_notify() {
-                var now = jsDateToDateTime((new Date())).getInTimezone("UTC");
+                var now = nowUTC();
                 var start;
                 if (!this.alarmService.mRangeEnd) {
                     // This is our first search for alarms.  We're going to look for
@@ -312,7 +324,7 @@ calAlarmService.prototype = {
                 // We don't set timers for every future alarm, only those within 6 hours
                 var end = now.clone();
                 end.hour += kHoursBetweenUpdates;
-                this.alarmService.mRangeEnd = end.getInTimezone("UTC");
+                this.alarmService.mRangeEnd = end.getInTimezone(UTC());
 
                 this.alarmService.findAlarms(this.alarmService.calendarManager.getCalendars({}),
                                              start, until);
@@ -368,7 +380,6 @@ calAlarmService.prototype = {
         this.mStarted = false;
     },
 
-
     observeCalendar: function cas_observeCalendar(calendar) {
         calendar.addObserver(this.calendarObserver);
     },
@@ -378,16 +389,43 @@ calAlarmService.prototype = {
         this.notifyObservers("onRemoveAlarmsByCalendar", [calendar]);
     },
 
-    addAlarm: function cas_addAlarm(aItem) {
-        var alarmTime;
+    getAlarmDate: function cas_getAlarmTime(aItem) {
+        var alarmDate = null;
         if (aItem.alarmRelated == Components.interfaces.calIItemBase.ALARM_RELATED_START) {
-            alarmTime = aItem.startDate || aItem.entryDate || aItem.dueDate;
+            alarmDate = aItem.startDate || aItem.entryDate || aItem.dueDate;
         } else {
-            alarmTime = aItem.endDate || aItem.dueDate || aItem.entryDate;
+            alarmDate = aItem.endDate || aItem.dueDate || aItem.entryDate;
         }
 
+        if (!aItem.alarmOffset && !aItem.parentItem.alarmOffset || !alarmDate) {
+            // If there is no alarm offset, or no date the alarm offset could be
+            // relative to, then there is no valid alarm.
+            return null;
+        }
+
+        alarmDate = alarmDate.clone();
+
+        // Handle all day events.  This is kinda weird, because they don't have
+        // a well defined startTime.  We just consider the start/end to be
+        // midnight in the user's timezone.
+        if (alarmDate.isDate) {
+            alarmDate = alarmDate.getInTimezone(this.timezone);
+            alarmDate.isDate = false;
+        }
+
+        var offset = aItem.alarmOffset || aItem.parentItem.alarmOffset;
+
+        alarmDate.addDuration(offset);
+        alarmDate = alarmDate.getInTimezone(UTC());
+
+        return alarmDate;
+    },
+
+    addAlarm: function cas_addAlarm(aItem) {
+        // Get the alarm time
+        var alarmTime = this.getAlarmDate(aItem);
         if (!alarmTime) {
-            ASSERT(false, "[calAlarmService] Could not determine alarm time for item: " + aItem.title);
+            // If there is no alarm time, don't add the alarm.
             return;
         }
 
@@ -404,31 +442,18 @@ calAlarmService.prototype = {
             time.icalString = snoozeTime;
             snoozeTime = time;
         }
-        LOG("[calAlarmService] snooze time: " + snoozeTime);
-        alarmTime = alarmTime.clone();
-
-        // Handle all day events.  This is kinda weird, because they don't have
-        // a well defined startTime.  We just consider the start/end to be 
-        // midnight in the user's timezone.
-        if (alarmTime.isDate) {
-            alarmTime = alarmTime.getInTimezone(this.mTimezone);
-            alarmTime.isDate = false;
-        }
-
-        var offset = aItem.alarmOffset || aItem.parentItem.alarmOffset;
-
-        alarmTime.addDuration(offset);
-        alarmTime = alarmTime.getInTimezone("UTC");
-        alarmTime = snoozeTime || alarmTime;
         LOG("[calAlarmService] considering alarm for item: " + aItem.title +
-            "\n offset: " + offset +
-            ", which makes alarm time: " + alarmTime);
-        var now = jsDateToDateTime((new Date()));
-        if (alarmTime.timezone == "floating") {
+            " alarm time: " + alarmTime + " snooze time: " + snoozeTime);
+
+        // If the alarm was snoozed, the snooze time is more important.
+        alarmTime = snoozeTime || alarmTime;
+
+        var now = jsDateToDateTime(new Date());
+        if (alarmTime.timezone.isFloating) {
             now = now.getInTimezone(calendarDefaultTimezone());
-            now.timezone = "floating";
+            now.timezone = floating();
         } else {
-            now = now.getInTimezone("UTC");
+            now = now.getInTimezone(UTC());
         }
         LOG("[calAlarmService] now is " + now);
         var callbackObj = {
@@ -471,6 +496,12 @@ calAlarmService.prototype = {
     },
 
     removeAlarm: function cas_removeAlarm(aItem) {
+        // If the item has no alarm start date, then it was never added so don't
+        // remove it.
+        if (!this.getAlarmDate(aItem)) {
+            return;
+        }
+
         // make sure already fired alarms are purged out of the alarm window:
         this.notifyObservers("onRemoveAlarmsByItem", [aItem]);
         for each (var timer in this.removeTimers(aItem)) {
@@ -492,7 +523,7 @@ calAlarmService.prototype = {
             cal[aItem.id] = itemTimers;
         }
         var rid = aItem.recurrenceId;
-        itemTimers[rid ? rid.getInTimezone("UTC").icalString : "mTimer"] = aTimer;
+        itemTimers[rid ? rid.getInTimezone(UTC()).icalString : "mTimer"] = aTimer;
         ++itemTimers.mCount;
     },
 
@@ -503,7 +534,7 @@ calAlarmService.prototype = {
             if (itemTimers) {
                 var rid = aItem.recurrenceId;
                 if (rid) {
-                    rid = rid.getInTimezone("UTC").icalString;
+                    rid = rid.getInTimezone(UTC()).icalString;
                     var timer = itemTimers[rid];
                     if (timer) {
                         delete itemTimers[rid];
@@ -535,9 +566,7 @@ calAlarmService.prototype = {
                     // assure we don't fire alarms twice, handle removed alarms as far as we can:
                     // e.g. we cannot purge removed items from ics files. XXX todo.
                     this.alarmService.removeAlarm(item);
-                    if (this.alarmService.hasAlarm(item)) {
-                        this.alarmService.addAlarm(item);
-                    }
+                    this.alarmService.addAlarm(item);
                 }
             }
         };
@@ -549,7 +578,8 @@ calAlarmService.prototype = {
 
         for each(var calendar in calendars) {
             // assuming that suppressAlarms does not change anymore until refresh:
-            if (!calendar.suppressAlarms) {
+            if (!calendar.getProperty("suppressAlarms") &&
+                calendar.getProperty("capabilities.alarms.popup.supported") !== false) {
                 calendar.getItems(filter, 0, start, until, getListener);
             }
         }
@@ -560,7 +590,7 @@ calAlarmService.prototype = {
         // alarms +/- 1 month from now.  If someone sets an alarm more than
         // a month ahead of an event, or doesn't start Sunbird/Lightning
         // for a month, they'll miss some, but that's a slim chance
-        var start = jsDateToDateTime((new Date())).getInTimezone("UTC");
+        var start = nowUTC();
         var until = start.clone();
         start.month -= 1;
         until.month += 1;
@@ -568,9 +598,10 @@ calAlarmService.prototype = {
     },
 
     alarmFired: function cas_alarmFired(event) {
-        if (event.calendar.suppressAlarms)
+        if (event.calendar.getProperty("suppressAlarms") ||
+            event.calendar.getProperty("capabilities.alarms.popup.supported") === false) {
             return;
-
+        }
         this.notifyObservers("onAlarm", [event]);
     }
 };
